!**********************************************************************
! file mane   : distatmod.f90 (version 1.4)
! programmer  : makoto nakajima (makoto@uiuc.edu)
! date        : february 14, 2007
! history     : (initial) part of module maklibrary and named
!                         mod_dist_stats
!               (ver 1.0) first independent version
!               (ver 1.1) added dist_doall and other convenient
!                         all-in-one subroutines
!               (ver 1.2) expanded dist_doall and added dist_doallout
!               (ver 1.3) added some stats.
!               (ver 1.4) clean-up.
! description : module to compute various distribution statistics.
! copyright   : 2004-2007 makoto nakajima
!*********************************************************************
module distatmod

	!******* implicit variable declaration prohibited *******
	implicit none

	!******* default is private *******
	private

	!******* followings can be called from outside *******
	public:: dist_stats,dist_sort,dist_output,simp_stat
	public:: dist_doall,dist_doallout

contains

!*********************************************************************
! subroutine: dist_output
! Subroutin to write the distributional statistics computed by
! dist_stats to a file.
!*********************************************************************
subroutine dist_output(tempstat, fnum)
	implicit none

	!******* parameters *******
	integer,parameter:: ntempstat=30
	character(len=10),dimension(ntempstat),parameter:: statname=&
	(/'mean      ','median    ','mean/medi ','variance  ',&
	'std dev   ','coef var  ','1st quint ','2nd quint ',&
	'3rd quint ','4th quint ','5th quint ','80-95%    ',&
	'top 5%    ','top 1%    ','gini index','% positive',&
	'% zero    ','% negative','90/10ratio','90/50ratio',&
	'50/10ratio','max       ','min       ','0-20%     ',&
	'0-40%     ','0-60%     ','0-80%     ','0-95%     ',&
	'0-99%     ','total meas'/)

	!******* input/output variables *******
	integer,intent(in):: fnum
	real(8),intent(in),dimension(ntempstat):: tempstat

	!******* local variables *******
	integer:: stat0

	do stat0=1,ntempstat
		write(fnum,"(a10,'=',1x,f30.15)") statname(stat0),tempstat(stat0)
	end do

end subroutine dist_output

!*********************************************************************
! subroutine: dist_doall
! Subroutine which combines all the procedures in this module.
!*********************************************************************
subroutine dist_doall(nid,idval,idmea,opt)
	implicit none

	!******* parameters *******
	integer,parameter:: ntempstat=30

	!******* input/output variables *******
	integer,intent(in):: nid,opt
	real(8),intent(in), dimension(nid):: idval,idmea

	!******* local variables *******
	real(8),dimension(ntempstat):: tempstat
	integer,dimension(nid):: idnum
	real(8),dimension(nid):: idvaltemp,idmeatemp

	idvaltemp(:)=idval(:)
	idmeatemp(:)=idmea(:)

	call dist_sort(nid,idnum,idvaltemp,idmeatemp)
	call dist_stats(nid,idvaltemp,idmeatemp,tempstat)
	if (opt>0) call dist_output(tempstat,opt)

end subroutine dist_doall

!*********************************************************************
! subroutine: dist_doallout
! dist_doitall plus returns the statistics
!*********************************************************************
subroutine dist_doallout(nid,idval,idmea,tempstat,opt)
	implicit none

	!******* parameters *******
	integer,parameter:: ntempstat=30

	!******* input/output variables *******
	integer,intent(in):: nid,opt
	real(8),intent(in), dimension(nid):: idval,idmea
	real(8),intent(out),dimension(ntempstat):: tempstat

	!******* local variables *******
	integer,dimension(nid):: idnum
	real(8),dimension(nid):: idvaltemp,idmeatemp

	idvaltemp(:)=idval(:)
	idmeatemp(:)=idmea(:)

	call dist_sort(nid,idnum,idvaltemp,idmeatemp)
	call dist_stats(nid,idvaltemp,idmeatemp,tempstat)
	if (opt>0) call dist_output(tempstat,opt)

end subroutine dist_doallout

!*********************************************************************
! subroutine: simp_stat
! same as dist_doall, but all data carry the same weight.
!*********************************************************************
subroutine simp_stat(nid,idval,opt)
	implicit none

	!******* parameters *******
	integer,parameter:: ntempstat=30

	!******* input/output variables *******
	integer,intent(in):: nid,opt
	real(8),intent(in), dimension(nid):: idval

	!******* local variables *******
	real(8),dimension(ntempstat):: tempstat
	integer,dimension(nid):: idnum
	real(8),dimension(nid):: idvaltemp,idmea

	idvaltemp(:)=idval(:)
	idmea(:)=1.0_8

	call dist_sort(nid,idnum,idvaltemp,idmea)
	call dist_stats(nid,idvaltemp,idmea,tempstat)
	if (opt>0) call dist_output(tempstat,opt)

end subroutine simp_stat

!*********************************************************************
! subroutine: dist_sort
! Subroutin to sort the data before computing distributional
! statistics, using heapsort.
!*********************************************************************
subroutine dist_sort(nid,idnum,idval,idmea)
	implicit none

	!******* parameters *******
	integer,parameter:: ntempstat=30

	!******* input/output variables *******
	integer,intent(in):: nid
	integer,intent(out),dimension(nid):: idnum
	real(8),intent(inout),dimension(nid):: idval
	real(8),intent(inout),dimension(nid):: idmea

	!******* local variables *******
	integer:: id0
	real(8),dimension(nid):: rtemp

	!******* sort idval *******
	call heapsort(nid, idval(1:nid), idnum(1:nid))

	!******* sort other data *******
	do id0=1, nid
		rtemp(id0)=idmea(idnum(id0))
	end do
	idmea(1:nid)=rtemp(1:nid)

end subroutine dist_sort

!********************************************************************
! subroutine: dist_stats
!********************************************************************
subroutine dist_stats(nid,idval,idmea,tempstats)
	implicit none

	!******* parameters *******
	integer,parameter:: ntempstat=30
	real(8),parameter:: smallreal=1.0e-30

	!******* input/output variables *******
	integer,intent(in):: nid
	real(8),intent(in), dimension(nid):: idval
	real(8),intent(in), dimension(nid):: idmea
	real(8),intent(out),dimension(ntempstat) :: tempstats

	!******* local variables *******
	integer:: pc0,id0
	real(8):: totsum,gini,leftsum,rightsum,left45,right45,triarea
	real(8):: idmean,idmedian,idcoefvar,idmeasum,id10,id90
	real(8),dimension(100):: pcprop, pcmea
	real(8),dimension(nid):: idtemp

	!******* initializtion *******
	idmeasum=sum(idmea(1:nid))
	if (idmeasum<smallreal) then
		tempstats(1:ntempstat)=0.0_8
		return
	end if
	idtemp(1:nid)=idmea(1:nid)/idmeasum
	pcmea(1:100)=0.010_8
	pcprop(1:100)=0.0_8

	!******* calculate mean, median, ratio *******
	idmean=0.0_8
	do id0=1,nid
		idmean=idmean+idval(id0)*idmea(id0)/idmeasum
	end do
	do id0=1,nid
		if (sum(idmea(1:id0))/idmeasum>=0.50_8) exit
		if (id0==nid) exit
	end do
	idmedian=idval(id0)
	tempstats(1)=idmean
	tempstats(2)=idmedian
	if (idmedian>smallreal) then
		tempstats(3)=idmean/idmedian
	else
		tempstats(3)=0.0_8
	end if

	!******* calculate coefficient of variation *******
	idcoefvar=0.0_8
	do id0=1, nid
		idcoefvar=idcoefvar+(idval(id0)-idmean)**2.0_8&
		*idmea(id0)/idmeasum
	end do
	tempstats(4)=idcoefvar
	idcoefvar=idcoefvar**0.50_8
	tempstats(5)=idcoefvar
	if (idmean>smallreal) then
		idcoefvar=idcoefvar/idmean
	else
		idcoefvar=0.0_8
	end if
	tempstats(6)=idcoefvar

	!******* loop *******
	id0=1
	do pc0=1,100
		do
			if (id0>nid) exit
			if (idtemp(id0)>=pcmea(pc0)) then
				pcprop(pc0)=pcprop(pc0)+pcmea(pc0)*idval(id0)
				idtemp(id0)=idtemp(id0)-pcmea(pc0)
				pcmea(pc0)=0.0_8
				exit
			else
				pcprop(pc0)=pcprop(pc0)+idtemp(id0)*idval(id0)
				pcmea(pc0)=pcmea(pc0)-idtemp(id0)
				idtemp(id0)=0.0_8
				id0=id0+1
			end if
		end do
	end do

	!******* calculate statistics *******
	totsum=sum(pcprop(1:100))
	tempstats(7)=sum(pcprop( 1:20))/totsum
	tempstats(8)=sum(pcprop(21:40))/totsum
	tempstats(9)=sum(pcprop(41:60))/totsum
	tempstats(10)=sum(pcprop(61:80))/totsum
	tempstats(11)=sum(pcprop(81:100))/totsum
	tempstats(12)=sum(pcprop(81:95))/totsum
	tempstats(13)=sum(pcprop(96:100))/totsum
	tempstats(14)=sum(pcprop(100:100))/totsum

	!******* calculate gini *******
	gini=0.0_8
	triarea=0.0_8
	do pc0=1,100
		if (pc0==1) then
			leftsum=0.0_8
		else
			leftsum=sum(pcprop(1:pc0-1))/totsum
		end if
		rightsum=sum(pcprop(1:pc0))/totsum
		left45=real(pc0-1,8)/100.0_8
		right45=real(pc0,8)/100.0_8
		gini=gini+((left45+right45)-(leftsum+rightsum))&
		/100.0_8/2.0_8
		triarea=triarea+(left45+right45)/100.0_8/2.0_8
	end do
	tempstats(15)=gini/triarea

	!******* calculate proportion with pos, zero, and neg *******
	tempstats(16:18)=0.0_8
	do id0=1,nid
		if (idval(id0)>0.0_8) then
			tempstats(16)=tempstats(16)+idmea(id0)
		else if (idval(id0)==0.0_8) then
			tempstats(17)=tempstats(17)+idmea(id0)
		else
			tempstats(18)=tempstats(18)+idmea(id0)
		end if
	end do
	tempstats(16)=tempstats(16)/max(sum(idmea(1:nid)),1.0d-30)
	tempstats(17)=tempstats(17)/max(sum(idmea(1:nid)),1.0d-30)
	tempstats(18)=tempstats(18)/max(sum(idmea(1:nid)),1.0d-30)

	!******* calculate 90/10, 90/50, 50/10 ratio *******
	do id0=1,nid
		if (sum(idmea(1:id0))/idmeasum>=0.10_8) exit
		if (id0==nid) exit
	end do
	id10=idval(id0)
	do id0=1, nid
		if (sum(idmea(1:id0))/idmeasum>=0.90_8) exit
		if (id0==nid) exit
	end do
	id90=idval(id0)
	if (id10>1.0e-30) then
		tempstats(19)=id90/id10
		tempstats(20)=idmedian/id10
	else
		tempstats(19)=0.0_8
		tempstats(20)=0.0_8
	end if
	if (idmedian>smallreal) then
		tempstats(21)=id90/idmedian
	else
		tempstats(21)=0.0_8
	end if

	!******* compute max and min *******
	do id0=nid,1,-1
		if (idmea(id0)>0.0_8) exit
		if (id0==1) exit
	end do
	tempstats(22)=idval(id0)
	do id0=1,nid
		if (idmea(id0)>0.0_8) exit
		if (id0==nid) exit
	end do
	tempstats(23)=idval(id0)

	!******* compute stats for lorenz *******
	tempstats(24)=tempstats( 7)
	tempstats(25)=sum(tempstats(7:8))
	tempstats(26)=sum(tempstats(7:9))
	tempstats(27)=sum(tempstats(7:10))
	tempstats(28)=sum(tempstats(7:10))+tempstats(12)
	tempstats(29)=1.0_8-tempstats(14)

	!******* total measure *******
	tempstats(30)=sum(idmea(1:nid))

end subroutine dist_stats

!********************************************************************
! subroutine : heapsort
! comments   : taken from "numerical recipes" in fortran 77, 2nd ed
!********************************************************************
subroutine heapsort(n,ra,order)
	implicit none

	!******* input/output variables *******
	integer,intent(in):: n
	real(8),intent(inout), dimension(n):: ra
	integer,intent(out), dimension(n):: order

	!******* local variables *******
	integer:: i, ir, j, l, rorder
	real(8):: rra

	if (n<2) return
	do l=1,n
		order(l)=l
	end do
	l=n/2+1
	ir=n
10 continue
	if (l>1) then
		l=l-1
		rra=ra(l); rorder=order(l)
	else
		rra=ra(ir); rorder=order(ir)
		ra(ir)=ra(1); order(ir)=order(1)
		ir=ir-1
		if (ir==1) then
			ra(1)=rra; order(1)=rorder
			return
		end if
	end if
	i=l
	j=l+l
20 continue
	if (j<=ir) then
		if (j<ir) then
			if (ra(j)<ra(j+1)) j=j+1
		end if
		if (rra<ra(j)) then
			ra(i)=ra(j); order(i)=order(j)
			i=j
			j=j+j
		else
			j=ir+1
		end if
		goto 20
	end if
	ra(i)=rra; order(i)=rorder
	goto 10

end subroutine heapsort

end module distatmod
