!*********************************************************************
! file name   : gridmod.f90 (ver 1.6)
! programmer  : makoto nakajima (makoto@uiuc.edu)
! date        : february 14, 2007
! history     : (ver 1.0) initial version.
!               (ver 1.1) cleaned up and changed names.
!               (ver 1.2) added print-out option.
!               (ver 1.3) added setgrid2 and setgrid3.
!               (ver 1.4) added subsum
!               (ver 1.5) added setgrid3n
!               (ver 1.6) severed subsum and cleaned up.
! description : subroutines for creating grid points.
! copyright   : 2007 makoto nakajima
!*********************************************************************
module gridmod

	!******* implicit variable declaration prohibited *******
	implicit none

	!******* default is private *******
	private

	!******* followings can be called from outside *******
	public:: setgrid0  !equally-spaced grids
	public:: setgrid1  !log-equally-spaced grids
	public:: setgrid2  !log-equally-spaced grids (unit can be specified)
	public:: setgrid3  !log-equally-spaced grids (unit automatically set)
	public:: setgrid3n !setgrid3 flipped for negative grids

contains

!*********************************************************************
! subroutine  : setgrid0
! description : construct equally-spaced grids
!*********************************************************************
subroutine setgrid0(ngrid,lbgrid,ubgrid,gridval,opt)
	implicit none

	!******* input/output variables *******
	integer,intent(in):: ngrid,opt
	real(8),intent(in):: lbgrid,ubgrid
	real(8),dimension(ngrid),intent(out):: gridval

	!******* local variables *******
	integer:: grid0

	!******* set grids *******
	if (ngrid==1) then
		gridval(1)=(lbgrid+ubgrid)/2.0_8
		return
	end if
	gridval(    1)=lbgrid
	gridval(ngrid)=ubgrid
	if (ngrid==2) return
	do grid0=2, ngrid-1
		gridval(grid0)=lbgrid+(ubgrid-lbgrid)*&
		real(grid0-1,8)/real(ngrid-1,8)
	end do

	!******* print out *******
	if (opt>0) then
		write(opt,"('------- setgrid0: result -------')")
		call printgrid(ngrid,gridval,opt)
	end if

end subroutine setgrid0

!*********************************************************************
! subroutine  : setgrid1
! description : construct log-equally-spaced grids
!*********************************************************************
subroutine setgrid1(ngrid,lbgrid,ubgrid,gridval,opt)
	implicit none

	!******* input/output variables *******
	integer,intent(in):: ngrid,opt
	real(8),intent(in):: lbgrid,ubgrid
	real(8),dimension(ngrid),intent(out):: gridval

	!******* local variables *******
	integer:: grid0

	!******* set grids *******
	if (ngrid==1) then
		gridval(1)=(lbgrid+ubgrid)/2.0_8
		return
	else if (ngrid==2) then
		gridval(    1)=lbgrid
		gridval(ngrid)=ubgrid
		return
	end if
	gridval(    1)=log(1.0_8)
	gridval(ngrid)=log(1.0_8+ubgrid-lbgrid)
	do grid0=2,ngrid-1
		gridval(grid0)=gridval(1)&
		+(gridval(ngrid)-gridval(1))*real(grid0-1,8)/real(ngrid-1,8)
	end do
	do grid0=1,ngrid
		gridval(grid0)=lbgrid+exp(gridval(grid0))-1.0_8
	end do
	gridval(    1)=lbgrid
	gridval(ngrid)=ubgrid

	!******* print out *******
	if (opt>0) then
		write(opt,"('------- setgrid1: result -------')")
		call printgrid(ngrid,gridval,opt)
	end if

end subroutine setgrid1

!*********************************************************************
! subroutine  : setgrid2
! description : construct log-equally-spaced grids (unit specified)
!*********************************************************************
subroutine setgrid2(ngrid,unitgrid,lbgrid,ubgrid,gridval,opt)
	implicit none

	!******* input/output variables *******
	integer,intent(in):: ngrid,opt
	real(8),intent(in):: unitgrid,lbgrid,ubgrid
	real(8),dimension(ngrid),intent(out):: gridval

	!******* local variables *******
	integer:: grid0

	!******* set grids *******
	if (ngrid==1) then
		gridval(1)=(lbgrid+ubgrid)/2.0_8
		return
	else if (ngrid==2) then
		gridval(    1)=lbgrid
		gridval(ngrid)=ubgrid
		return
	end if
	gridval(    1)=log(1.0_8)
	gridval(ngrid)=log(1.0_8+ubgrid/unitgrid-lbgrid/unitgrid)
	do grid0=2,ngrid-1
		gridval(grid0)=gridval(1)&
		+(gridval(ngrid)-gridval(1))*real(grid0-1,8)/real(ngrid-1,8)
	end do
	do grid0=1,ngrid
		gridval(grid0)=(lbgrid/unitgrid+exp(gridval(grid0))-1.0_8)*unitgrid
	end do
	gridval(    1)=lbgrid
	gridval(ngrid)=ubgrid

	!******* print out *******
	if (opt>0) then
		write(opt,"('------- setgrid2: result -------')")
		call printgrid(ngrid,gridval,opt)
	end if

end subroutine setgrid2

!*********************************************************************
! subroutine  : setgrid3
! description : construct log-equally-spaced grids
!               (unit is automatically computed).
!*********************************************************************
subroutine setgrid3(ngrid,lbgrid,ubgrid,gridval,opt)
	implicit none

	!******* input/output variables *******
	integer,intent(in):: ngrid,opt
	real(8),intent(in):: lbgrid,ubgrid
	real(8),dimension(ngrid),intent(out):: gridval

	!******* local variables *******
	real(8):: unitgrid
	integer:: grid0

	!******* set grids *******
	if (ngrid==1) then
		gridval(1)=(lbgrid+ubgrid)/2.0_8
		return
	else if (ngrid==2) then
		gridval(    1)=lbgrid
		gridval(ngrid)=ubgrid
		return
	end if
	unitgrid=(ubgrid-lbgrid)/real(ngrid-1,8)
	gridval(    1)=log(1.0_8)
	gridval(ngrid)=log(1.0_8+ubgrid/unitgrid-lbgrid/unitgrid)
	do grid0=2,ngrid-1
		gridval(grid0)=gridval(1)&
		+(gridval(ngrid)-gridval(1))*real(grid0-1,8)/real(ngrid-1,8)
	end do
	do grid0=1,ngrid
		gridval(grid0)=(lbgrid/unitgrid+exp(gridval(grid0))-1.0_8)*unitgrid
	end do
	gridval(    1)=lbgrid
	gridval(ngrid)=ubgrid

	!******* print out *******
	if (opt>0) then
		write(opt,"('------- setgrid3: result -------')")
		call printgrid(ngrid,gridval,opt)
	end if

end subroutine setgrid3

!*********************************************************************
! subroutine  : setgrid3n
!*********************************************************************
subroutine setgrid3n(ngrid,lbgrid,ubgrid,gridval,opt)
	implicit none

	!******* input/output variables *******
	integer,intent(in):: ngrid,opt
	real(8),intent(in):: lbgrid,ubgrid
	real(8),dimension(ngrid),intent(out):: gridval

	!******* local variables *******
	integer:: grid0
	real(8),dimension(ngrid):: gridvaltemp

	!******* set grids for lower part *******
	call setgrid3(ngrid,-ubgrid,-lbgrid,gridvaltemp,0)
	do grid0=1,ngrid
		gridval(grid0)=-gridvaltemp(ngrid+1-grid0)
	end do

	!******* print out *******
	if (opt>0) then
		write(opt,"('------- setgrid3n: result -------')")
		call printgrid(ngrid,gridval,opt)
	end if

end subroutine setgrid3n

!*********************************************************************
! subroutine  : printgrid
!*********************************************************************
subroutine printgrid(ngrid,gridval,opt)
	implicit none

	!******* input/output variables *******
	integer,intent(in):: ngrid,opt
	real(8),dimension(ngrid),intent(in):: gridval

	!******* local variables *******
	integer:: grid0

	!******* print out *******
	write(opt,"('  grid num',1x,'                    grid value')")
	do grid0=1,ngrid
		write(opt,"(i10,1x,f30.20)") grid0,gridval(grid0)
	end do
	write(opt,"('')")

end subroutine printgrid

end module gridmod
