!*********************************************************************
! file name   : markovmod.f90 (ver 1.4)
! programmer  : makoto nakajima (makoto@uiuc.edu)
! date        : february 14, 2007
! description : collection of subroutines related to markov chain.
! history     : (ver 1.0) initial version.
!               (ver 1.1) diststdv is added.
!               (ver 1.2) markovupdate is added.
!               (ver 1.3) ergodist prints out results.
!               (ver 1.4) clean-up
! copyright   : 2004-2007 makoto nakajima
!*********************************************************************
module markovmod

	!******* implicit variable declaration prohibited *******
	implicit none

	!******* default is private *******
	private

	!******* followings can be called from outside *******
	public:: ergodist,distmean,distmexp,diststdv,distsexp,distmom2
	public:: markovupdate

contains

!*********************************************************************
! subroutine  : ergodist
! description : finding ergodic distribution of Markov process
!*********************************************************************
subroutine ergodist(ns,stran,sdist,opt)
	implicit none
	integer,intent(in):: ns,opt
	real(8),intent(in),dimension(1:ns,1:ns):: stran
	real(8),intent(out),dimension(1:ns):: sdist
	real(8),dimension(ns):: sdist0,sdist1
	integer:: s0,s1,iter0,maxiter
	real(8):: eps,error0

	!******* set parameters *******
	eps=1.0e-10
	maxiter=1000000

	!******* find ergodic distribution *******
	sdist0(1:ns)=1.0_8/real(ns,8)
	do iter0=1,maxiter
		sdist1(:)=0.0_8
		do s0=1,ns
			do s1=1,ns
				sdist1(s1)=sdist1(s1)+sdist0(s0)*stran(s0,s1)
			end do
		end do
		error0=maxval(abs(sdist0(:)-sdist1(:)))
		if (error0<eps) exit
		if (iter0==maxiter) then
			if (opt>0) then
				write(opt,"('ergodic dist not found within ',i5)") maxiter
			end if
			stop
		end if
		sdist0=sdist1
	end do
	
	!******* set the output and return *******
	sdist(:)=sdist1(:)

	!******* print out *******
	if (opt>0) then
		write(opt,"('------- ergodist: result -------')")
		do s0=1,ns
			write(opt,"('s0=',i5,' dist=',f20.18)") s0,sdist(s0)
		end do
		write(opt,"('sum',5x,' dist=',f20.18,/)") sum(sdist(:))
	end if

	!******* end of subroutine *******
	return

end subroutine ergodist

!*********************************************************************
! subroutine  : distmean
! description : computing mean of distribution
!*********************************************************************
subroutine distmean(ns,sdist,sval,smean)
	implicit none
	integer,intent(in):: ns
	real(8),intent(in),dimension(1:ns):: sdist,sval
	real(8),intent(out):: smean
	integer:: s0

	smean=0.0_8
	do s0=1,ns
		smean=smean+sdist(s0)*sval(s0)
	end do
	return

end subroutine distmean

!*********************************************************************
! subroutine  : markovupdate
! description : updating distribution using markov chain
!*********************************************************************
subroutine markovupdate(ns,stran,sdist)
	implicit none
	integer,intent(in):: ns
	real(8),intent(in),dimension(1:ns,1:ns):: stran
	real(8),intent(inout),dimension(1:ns):: sdist
	real(8),dimension(1:ns):: sdisttemp
	integer:: s0,s1

	sdisttemp(:)=0.0_8
	do s0=1,ns
		do s1=1,ns
			sdisttemp(s1)=sdisttemp(s1)+sdist(s0)*stran(s0,s1)
		end do
	end do
	sdist(:)=sdisttemp(:)

end subroutine markovupdate

!*********************************************************************
! subroutine  : diststdv
! description : computing std dev of distribution
!*********************************************************************
subroutine diststdv(ns,sdist,sval,smean,sstdv)
	implicit none
	integer,intent(in):: ns
	real(8),intent(in),dimension(1:ns):: sdist,sval
	real(8),intent(in):: smean
	real(8),intent(out):: sstdv
	integer:: s0

	sstdv=0.0_8
	do s0=1,ns
		sstdv=sstdv+sdist(s0)*(sval(s0)-smean)**2.0_8
	end do
	sstdv=sqrt(sstdv)
	return

end subroutine diststdv

!*********************************************************************
! subroutine  : distmom2
! description : does both distmean and diststdv
!*********************************************************************
subroutine distmom2(ns,sdist,sval,smean,sstdv)
	implicit none
	integer,intent(in):: ns
	real(8),intent(in),dimension(1:ns):: sdist,sval
	real(8),intent(out):: smean,sstdv
	integer:: s0
	call distmean(ns,sdist,sval,smean)
	call diststdv(ns,sdist,sval,smean,sstdv)
	return
end subroutine distmom2

!*********************************************************************
! subroutine  : distmexp
! description : computing mean of exp of distribution
!*********************************************************************
subroutine distmexp(ns,sdist,sval,smean)
	implicit none
	integer,intent(in):: ns
	real(8),intent(in),dimension(1:ns):: sdist,sval
	real(8),intent(out):: smean
	integer:: s0

	smean=0.0_8
	do s0=1,ns
		smean=smean+sdist(s0)*exp(sval(s0))
	end do
	return

end subroutine distmexp

!*********************************************************************
! subroutine  : distsexp
! description : computing std dev of exp of distribution
!*********************************************************************
subroutine distsexp(ns,sdist,sval,smean,sstdv)
	implicit none
	integer,intent(in):: ns
	real(8),intent(in),dimension(1:ns):: sdist,sval
	real(8),intent(in):: smean
	real(8),intent(out):: sstdv
	integer:: s0

	sstdv=0.0_8
	do s0=1,ns
		sstdv=sstdv+sdist(s0)*(exp(sval(s0))-smean)**2.0_8
	end do
	sstdv=sqrt(sstdv)
	return

end subroutine distsexp

end module markovmod
