!*********************************************************************
! file name   : subsummod.f90 (ver 1.0)
! programmer  : makoto nakajima (makoto@uiuc.edu)
! date        : february 14, 2007
! history     : (ver 1.0) initial version.
! description : subroutines for computing sum of a variable owned
!               by a subset of agents.
! copyright   : 2007 makoto nakajima
!*********************************************************************
module subsummod

	!******* implicit variable declaration prohibited *******
	implicit none

	!******* default is private *******
	private

	!******* followings can be called from outside *******
	public:: subsum
	public:: subsum10
	public:: subsum20

contains

!*********************************************************************
! subroutine  : subsum
! description : construct sum held by subset of agents
!*********************************************************************
subroutine subsum(nall,allval,allmeain,sublb,subub,subtot)
	implicit none

	!******* input/output variables *******
	integer,intent(in):: nall
	real(8),intent(in),dimension(nall):: allval,allmeain
	real(8),intent(in):: sublb,subub
	real(8),intent(out):: subtot

	!******* local variables *******
	integer:: all0l,all0u,all0
	real(8),dimension(nall):: allmea

	!******* normalize measure *******
	allmea=allmeain/sum(allmeain)

	!******* reset output *******
	subtot=0.0_8

	!******* check appropriate bounds *******
	if (sublb<0.0_8 .or. subub>1.0_8 .or. sublb>subub) return

	!******* find lower-end grid *******
	do all0l=1,nall
		if (sum(allmea(1:all0l))>=sublb) exit
	end do

	!******* find upper-end grid *******
	do all0u=1,nall
		if (sum(allmea(1:all0u))>=subub) exit
	end do

	!******* special case: all0l==all0u *******
	if (all0l==all0u) then
		subtot=(subub-sublb)*allval(all0l)/allmea(all0l)
		return
	end if

	!******* process lower-end *******
	subtot=subtot+(sum(allmea(1:all0l))-sublb)&
	*allval(all0l)/allmea(all0l)

	!******* process upper-end *******
	subtot=subtot+(subub-sum(allmea(1:all0u-1)))&
	*allval(all0u)/allmea(all0u)

	!******* done if all0u==all0l+1 *******
	if (all0u==all0l+1) return

	!******* process middle *******
	do all0=all0l+1,all0u-1
		subtot=subtot+allval(all0)
	end do

end subroutine subsum

!*********************************************************************
! subroutine  : subsum20
! description : construct sum held by each quintile
!*********************************************************************
subroutine subsum20(nsub,subrnkin,submeain,subvalin,substat)
	implicit none

	!******* input/output variables *******
	integer,intent(in):: nsub
	real(8),intent(in),dimension(nsub):: subrnkin,submeain,subvalin
	real(8),intent(out),dimension(6):: substat

	!******* local variables *******
	integer,dimension(nsub):: suborder
	real(8),dimension(nsub):: subrnk,submea,subval

	!******* reorder *******
	subrnk=subrnkin
	submea=submeain
	subval=subvalin
	call heapsort(nsub,subrnk,suborder)
	call reorder(nsub,subval,suborder)
	call reorder(nsub,submea,suborder)

	!******* call subsum for each quintile *******
	call subsum(nsub,subval,submea,0.00_8,0.20_8,substat(1))
	call subsum(nsub,subval,submea,0.20_8,0.40_8,substat(2))
	call subsum(nsub,subval,submea,0.40_8,0.60_8,substat(3))
	call subsum(nsub,subval,submea,0.60_8,0.80_8,substat(4))
	call subsum(nsub,subval,submea,0.80_8,1.00_8,substat(5))
	call subsum(nsub,subval,submea,0.00_8,1.00_8,substat(6))

end subroutine subsum20

!*********************************************************************
! subroutine  : subsum10
! description : construct sum held by each docie
!*********************************************************************
subroutine subsum10(nsub,subrnkin,submeain,subvalin,substat)
	implicit none

	!******* input/output variables *******
	integer,intent(in):: nsub
	real(8),intent(in),dimension(nsub):: subrnkin,submeain,subvalin
	real(8),intent(out),dimension(11):: substat

	!******* local variables *******
	integer,dimension(nsub):: suborder
	real(8),dimension(nsub):: subrnk,submea,subval

	!******* reorder *******
	subrnk=subrnkin
	submea=submeain
	subval=subvalin
	call heapsort(nsub,subrnk,suborder)
	call reorder(nsub,subval,suborder)
	call reorder(nsub,submea,suborder)

	!******* call subsum for each quintile *******
	call subsum(nsub,subval,submea,0.00_8,0.10_8,substat(1))
	call subsum(nsub,subval,submea,0.10_8,0.20_8,substat(2))
	call subsum(nsub,subval,submea,0.20_8,0.30_8,substat(3))
	call subsum(nsub,subval,submea,0.30_8,0.40_8,substat(4))
	call subsum(nsub,subval,submea,0.40_8,0.50_8,substat(5))
	call subsum(nsub,subval,submea,0.50_8,0.60_8,substat(6))
	call subsum(nsub,subval,submea,0.60_8,0.70_8,substat(7))
	call subsum(nsub,subval,submea,0.70_8,0.80_8,substat(8))
	call subsum(nsub,subval,submea,0.80_8,0.90_8,substat(9))
	call subsum(nsub,subval,submea,0.90_8,1.00_8,substat(10))
	call subsum(nsub,subval,submea,0.00_8,1.00_8,substat(11))

end subroutine subsum10

!*********************************************************************
! subroutine  :reorder
!*********************************************************************
subroutine reorder(nall,allval,allorder)
	implicit none

	!******* input/output variables *******
	integer,intent(in):: nall
	integer,intent(in),dimension(nall):: allorder
	real(8),intent(inout),dimension(nall):: allval

	!******* local variables *******
	integer:: all0
	real(8),dimension(nall):: alltemp

	do all0=1,nall
		alltemp(all0)=allval(allorder(all0))
	end do
	allval=alltemp

end subroutine reorder

!********************************************************************
! subroutine : heapsort
! comment    : taken from "numerical recipes" in fortan 77, 2nd ed
!********************************************************************
subroutine heapsort(n,ra,order)
	implicit none

	!******* input/output variables *******
	integer,intent(in):: n
	real(8),intent(inout), dimension(n):: ra
	integer,intent(out), dimension(n):: order

	!******* local variables *******
	integer:: i, ir, j, l, rorder
	real(8):: rra

	if (n<2) return
	do l=1,n
		order(l)=l
	end do
	l=n/2+1
	ir=n
10 continue
	if (l>1) then
		l=l-1
		rra=ra(l); rorder=order(l)
	else
		rra=ra(ir); rorder=order(ir)
		ra(ir)=ra(1); order(ir)=order(1)
		ir=ir-1
		if (ir==1) then
			ra(1)=rra; order(1)=rorder
			return
		end if
	end if
	i=l
	j=l+l
20 continue
	if (j<=ir) then
		if (j<ir) then
			if (ra(j)<ra(j+1)) j=j+1
		end if
		if (rra<ra(j)) then
			ra(i)=ra(j); order(i)=order(j)
			i=j
			j=j+j
		else
			j=ir+1
		end if
		goto 20
	end if
	ra(i)=rra; order(i)=rorder
	goto 10

end subroutine heapsort

end module subsummod
